import httpx



RUN_API_URL_FORMAT = "https://glot.io/run/{}?version=latest"
SUPPORTED_LANGUAGES = {
    "assembly": {"ext": "asm"},
    "bash": {"ext": "sh"},
    "c": {"ext": "c"},
    "clojure": {"ext": "clj"},
    "coffeescript": {"ext": "coffe"},
    "cpp": {"ext": "cpp"},
    "csharp": {"ext": "cs"},
    "erlang": {"ext": "erl"},
    "fsharp": {"ext": "fs"},
    "go": {"ext": "go"},
    "groovy": {"ext": "groovy"},
    "haskell": {"ext": "hs"},
    "java": {"ext": "java", "name": "Main"},
    "javascript": {"ext": "js"},
    "julia": {"ext": "jl"},
    "kotlin": {"ext": "kt"},
    "lua": {"ext": "lua"},
    "perl": {"ext": "pl"},
    "php": {"ext": "php"},
    "python": {"ext": "py"},
    "ruby": {"ext": "rb"},
    "rust": {"ext": "rs"},
    "scala": {"ext": "scala"},
    "swift": {"ext": "swift"},
    "typescript": {"ext": "ts"},
}


class CodeRunner():

    @staticmethod
    def help() -> str:
        return (
            "/code {语言}\n"
            "{代码}\n"
            "For example:\n"
            "/code python\n"
            "print('hello world')\n\n"
            "发送 /code -ls 查看支持的语言"
        )

    @staticmethod
    def list_supp_lang() -> str:
        msg0 = "咱现在支持的语言如下：\n"
        msg0 += ", ".join(map(str, SUPPORTED_LANGUAGES.keys()))
        return msg0

    @staticmethod
    async def runner(msg: str):
        args = msg.split("\n")

        if not args or len(args) == 1:
            return "请检查输入内容...发送 /code -h 查看帮助"

        if (lang := args[0].strip()) not in SUPPORTED_LANGUAGES:
            return "该语言暂不支持...或者可能格式错误？"

        del args[0]
        code = "\n".join(map(str, args))
        url = RUN_API_URL_FORMAT.format(lang)
        js = {
            "files": [
                {
                    "name": (
                        SUPPORTED_LANGUAGES[lang].get("name", "main")
                        + f".{SUPPORTED_LANGUAGES[lang]['ext']}"
                    ),
                    "content": code,
                }
            ],
            "stdin": "",
            "command": "",
        }

        try:
            res = httpx.post(url, json=js)
        except Exception as e:
            return repr(e)

        payload = res.json()
        res.close()

        sent = False
        for k in ["stdout", "stderr", "error"]:
            v = payload.get(k)
            lines = v.splitlines()
            lines, remained_lines = lines[:10], lines[10:]
            out = "\n".join(lines)
            out, remained_out = out[: 60 * 10], out[60 * 10 :]

            if remained_lines or remained_out:
                out += f"\n（太多了太多了...）"

            if out:
                return f"{k}:\n{out}"

        if not sent:
            return "运行完成，没任何输出呢..."



from telegram import Update
from telegram.ext import (
    ContextTypes,
    CommandHandler,
    filters
)

from remilia.log import logger
from remilia.config import NICKNAME


def run(application):
    start_handler = CommandHandler(
        'start', start,
        filters.ChatType.PRIVATE
    )
    application.add_handler(start_handler)

    start1_handler = CommandHandler(
        'start', start1,
        ~filters.ChatType.PRIVATE
    )
    application.add_handler(start1_handler)


async def start(update: Update, context: ContextTypes.DEFAULT_TYPE):
    logger.info(f'[{update.message.chat.type.upper()}]({update.message.chat_id}) {context._user_id}: {update.message.text}')
    await update.message.reply_text(
        f'Hello, This is {NICKNAME[0]}.\n'
        f'Your Telegram ID is: {context._user_id}'
    )


async def start1(update: Update, context: ContextTypes.DEFAULT_TYPE):
    logger.info(f'[{update.message.chat.type.upper()}]({update.message.chat_id}) {context._user_id}: {update.message.text}')
    await update.message.reply_text(f'The Chat ID is: {update.message.chat_id}')



from html import unescape

from telegram import Update
from telegram.ext import (
    ContextTypes,
    CommandHandler,
    filters
)

from remilia.log import logger
from .data_source import CodeRunner


def run(application):
    code_runner_handler = CommandHandler(
        'code', code_runner,
        filters.TEXT
    )
    application.add_handler(code_runner_handler)


async def code_runner(update: Update, context: ContextTypes.DEFAULT_TYPE):
    logger.info(f'[{update.message.chat.type.upper()}]({update.message.chat_id}) {context._user_id}: {update.message.text}')
    opt = (
        context.args[0] + update.message.text.split(context.args[0], 1)[1]
        if context.args else ''
    )
    if not opt:
        content = '发送 /code -h 查看帮助'
    elif opt == '-h':
        content = CodeRunner().help()
    elif opt == '-ls':
        content = CodeRunner().list_supp_lang()
    else:
        content = str(await CodeRunner().runner(unescape(opt)))
    await update.message.reply_text(content)




from sys import exc_info
import httpx
from httpx import AsyncClient

from telegram import InputMediaPhoto

from remilia.log import logger
from remilia.config import SETU


r18 = SETU['r18']
pixproxy = SETU['pixproxy']


async def get_setu(tag=[], num=1) -> list:
    logger.info('loading...')
    
    async with AsyncClient() as client:
        req_url = 'https://api.lolicon.app/setu/v2'
        params = {
            'tag': tag,
            'r18': r18,
            'size': 'regular',
            'num': num if num < 11 else 1
        }
        try:
            res = await client.get(req_url, params=params, timeout=30)
        except httpx.HTTPError as e:
            logger.warning(e)
            return [f'API异常{e}', False]
        try:
            logger.debug(content := res.json()['data'])
            _ = content[0]

            content = [{
                'pid': i['pid'],
                'url': i['urls']['regular'],
                'caption': (
                    f'标题: {i["title"]}\n'
                    f'pid: {i["pid"]}\n'
                    f'画师: {i["author"]}\n'
                    f'标签: {", ".join(i["tags"])}'
                )
            } for i in content]

            pics, status = await down_pic(content)

            logger.success('complete.')

            # if len(content) == 1:
            #     return [content[0]['url'], 1, content[0]['caption']]

            # media = [
            #     InputMediaPhoto(media=i['url'], caption=i['caption'])
            #     for i in content
            # ]
            # return [media, 2]

            if not pics:
                return ['\n'.join(status), False]
            if len(pics) == 1:
                return [pics[0], 1, '\n'.join(status) if status else '']

            media = [
                InputMediaPhoto(media=i[0], caption=i[1])
                for i in pics
            ]
            return [media, 2]

        except httpx.ProxyError as e:
            logger.warning(e)
            return [f'代理错误: {e}', False]
        except IndexError as e:
            logger.warning(e)
            return [f'图库中没有搜到关于{tag}的图。', False]
        except:
            logger.warning(f'{exc_info()[0]}, {exc_info()[1]}')
            return [f'{exc_info()[0]} {exc_info()[1]}。', False]



async def down_pic(content) -> tuple[list, list]:
    async with AsyncClient() as client:
        headers = {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/109.0.0.0 Safari/537.36',
        }
        if not pixproxy:
            headers = {
                'Host': 'i.pximg.net',
                'Referer': 'https://www.pixiv.net/',
                'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/109.0.0.0 Safari/537.36'
            }
        pics, status = [], []
        for i in content:
            res = await client.get(
                url = (
                    i['url'].replace('https://i.pixiv.re', pixproxy)
                    if pixproxy
                    else i['url'].replace('i.pixiv.re', 'i.pximg.net')
                ),
                headers=headers,
                timeout=30
            )
            if res.status_code == 200:
                logger.success(f'获取图片 {i["pid"]} 成功')
                pics.append([res.content, i['caption']])
            else:
                logger.error(sc := f'获取图片 {i["pid"]} 失败: {res.status_code}')
                status.append(sc)
            await res.aclose()
        return pics, status



from typing import Literal
from pathlib import Path
import ujson as json

from telegram import Update
from telegram.ext import (
    ContextTypes,
    CommandHandler,
    MessageHandler,
    filters
)

from remilia.log import logger
from remilia.config import SUPERUSERS


file_path = Path() / 'data' / 'chatlist' / 'chatlist.json'
file_path.parent.mkdir(parents=True, exist_ok=True)

# 白名单
chatlist = (
    json.loads(file_path.read_text('utf-8'))
    if file_path.is_file()
    else {'chats': [], 'users': []}
)

CHATS = filters.Chat(chatlist['chats'])
USERS = filters.User(chatlist['users'])


def run(application):

    wake_handler = CommandHandler(
        'wake', wake,
        filters.User(SUPERUSERS))
    application.add_handler(wake_handler)

    sleep_handler = CommandHandler(
        'sleep', sleep,
        filters.User(SUPERUSERS))
    application.add_handler(sleep_handler)

    banUser_handler = CommandHandler(
        'dwu', banUser,
        filters.User(SUPERUSERS))
    application.add_handler(banUser_handler)

    unbanUser_handler = CommandHandler(
        'awu', unbanUser,
        filters.User(SUPERUSERS))
    application.add_handler(unbanUser_handler)

    banChat_handler = CommandHandler(
        'dwc', banChat,
        filters.User(SUPERUSERS))
    application.add_handler(banChat_handler)

    unbanChat_handler = CommandHandler(
        'awc', unbanChat,
        filters.User(SUPERUSERS))
    application.add_handler(unbanChat_handler)


    block_handler = MessageHandler(~(CHATS|USERS), block)
    application.add_handler(block_handler)


async def wake(update: Update, context: ContextTypes.DEFAULT_TYPE):
    logger.info(f'[{update.message.chat.type.upper()}]({update.message.chat_id}) {context._user_id}: {update.message.text}')
    handle_chatlist([update.message.chat_id,], 'add', 'chats')
    await update.message.chat.send_message('呜......醒来力...')


async def sleep(update: Update, context: ContextTypes.DEFAULT_TYPE):
    logger.info(f'[{update.message.chat.type.upper()}]({update.message.chat_id}) {context._user_id}: {update.message.text}')
    handle_chatlist([update.message.chat_id,], 'del', 'chats')
    await update.message.chat.send_message('那我先去睡觉了...')


async def banUser(update: Update, context: ContextTypes.DEFAULT_TYPE):
    logger.info(f'[{update.message.chat.type.upper()}]({update.message.chat_id}) {context._user_id}: {update.message.text}')
    msg = handle_msg(context.args, 'del', 'users')
    await update.message.reply_text(msg)


async def unbanUser(update: Update, context: ContextTypes.DEFAULT_TYPE):
    logger.info(f'[{update.message.chat.type.upper()}]({update.message.chat_id}) {context._user_id}: {update.message.text}')
    msg = handle_msg(context.args, 'add', 'users')
    await update.message.reply_text(msg)


async def banChat(update: Update, context: ContextTypes.DEFAULT_TYPE):
    logger.info(f'[{update.message.chat.type.upper()}]({update.message.chat_id}) {context._user_id}: {update.message.text}')
    msg = handle_msg(context.args, 'del', 'chats')
    await update.message.reply_text(msg)


async def unbanChat(update: Update, context: ContextTypes.DEFAULT_TYPE):
    logger.info(f'[{update.message.chat.type.upper()}]({update.message.chat_id}) {context._user_id}: {update.message.text}')
    msg = handle_msg(context.args, 'add', 'chats')
    await update.message.reply_text(msg)



async def block(update: Update, context: ContextTypes.DEFAULT_TYPE):
    return



def save_chatlist() -> None:
    file_path.write_text(json.dumps(chatlist), encoding='utf-8')


def is_number(s: str) -> bool:
    try:
        float(s)
        return True
    except ValueError:
        pass
    try:
        import unicodedata
        unicodedata.numeric(s)
        return True
    except (TypeError, ValueError):
        pass
    return False


def handle_msg(
    args,
    mode: Literal['add', 'del'],
    type_: Literal['users', 'chats'],
) -> str:
    if not args:
        return ('用法:\n/ban(/unban) uid uid1 uid2 ...\n'
            '/banc(/unbanc) uid uid1 uid2 ...')
    for uid in args:
        if not is_number(uid):
            return '参数错误, uid必须是数字..'
    args = [int(uid) for uid in args]
    msg = handle_chatlist(args, mode, type_)
    return msg


def handle_chatlist(
    args: list,
    mode: Literal['add', 'del'],
    type_: Literal['users', 'chats'],
) -> str:
    if mode == 'add':
        chatlist[type_].extend(args)
        chatlist[type_] = list(set(chatlist[type_]))
        _mode = '添加'

        if type_ == 'users': USERS.add_user_ids(args) 
        else: CHATS.add_chat_ids(args)

    else:
        chatlist[type_] = [uid for uid in chatlist[type_] if uid not in args]
        _mode = '删除'

        if type_ == 'users': USERS.remove_user_ids(args)
        else: CHATS.remove_chat_ids(args)

    save_chatlist()
    _type = '用户' if type_ == 'users' else '会话'

    return f"白名单{_mode} {len(args)} 个{_type}:\n{args}"




from telegram import Update
from telegram.ext import (
    ContextTypes,
    CommandHandler
)

from remilia.log import logger
from .utils import get_pix


def run(application):
    pix_handler = CommandHandler('pix', pix)
    application.add_handler(pix_handler)


async def pix(update: Update, context: ContextTypes.DEFAULT_TYPE):
    logger.info(f'[{update.message.chat.type.upper()}]({update.message.chat_id}) {context._user_id}: {update.message.text}')
    if (keyword := context.args) and is_number(keyword[0]):
        num = int(keyword[0])
        keyword.remove(keyword[0])
    else:
        keyword = context.args
        num = 1

    content = await get_pix(keyword=keyword, img=num)

    if content[1] == 2:
        try:
            await update.message.reply_media_group(
                media = content[0],
                read_timeout = 60,
                write_timeout = 60,
                connect_timeout = 60,
                pool_timeout = 60,
            )
        except Exception as e:
            await update.message.chat.send_message(repr(e))
    elif content[1]:
        await update.message.reply_photo(
            photo = content[0][0],
            caption = content[0][1] + content[2]
        )
    else:
        await update.message.reply_text(str(content[0]))


def is_number(s: str) -> bool:
    try:
        float(s)
        return True
    except ValueError:
        pass
    try:
        import unicodedata
        unicodedata.numeric(s)
        return True
    except (TypeError, ValueError):
        pass
    return False



import yaml
from pathlib import Path
from pydantic import BaseModel, Extra
from typing import List, Dict


class Config(BaseModel, extra=Extra.ignore):
    LOG_LEVEL: str = 'INFO'
    TOKEN: str = ''
    NICKNAME: List[str] = ['remilia']
    SUPERUSERS: List[int] = []
    SETU: Dict = {'r18': 2, 'pixproxy': ''}
    ACGGOV: Dict = {'token': 'apikey', 'pixproxy': ''}
    XIAOAI: Dict = {'mp3': False}


default_config = """

LOG_LEVEL: 'DEBUG'

TOKEN: ''  # Telegram Bot API Token

NICKNAME:  # Bot的昵称
- Remilia
- Remi
- remilia
- remi

SUPERUSERS:  # Bot主人的 Telegram ID, ID为纯数字, 不是自己设置的用户名
- 123456
- 654321

SETU:
  r18: 2  # 0为非R18，1为R18，2为混合（在库中的分类，不等同于作品本身的R18标识）
  pixproxy: ''
  # pximg图片代理, 需要填写前缀 https:// 或 http://
  # 留空为直连 i.pximg.net

ACGGOV:
  token: ''
  pixproxy: ''

XIAOAI: 
  mp3: false  # 当 mp3 为 true 时尝试使用语音发送

""".strip()


filepath = Path() / 'config.yml'


if not filepath.is_file():
    try:
        filepath.write_text(default_config, encoding='utf-8')
    except Exception as e:
        print('生成配置文件失败!\n'+repr(e))
    print('配置文件已生成, 请填写config.yml后重新启动Bot')
    exit(-1)


config = yaml.safe_load(filepath.read_text('utf-8'))

_config = Config.parse_obj(config)

LOG_LEVEL: str = _config.LOG_LEVEL
TOKEN: str = _config.TOKEN
NICKNAME: List[str] = _config.NICKNAME
SUPERUSERS: List[int] = _config.SUPERUSERS
SETU: Dict = _config.SETU
ACGGOV: Dict = _config.ACGGOV
XIAOAI: Dict = _config.XIAOAI



from sys import exc_info
import httpx, random
from httpx import AsyncClient

from telegram import InputMediaPhoto

from remilia.log import logger
from remilia.config import ACGGOV


token = ACGGOV['token']
pixproxy = ACGGOV['pixproxy']

headers = {
    'token': token if token else 'apikey', 
    'referer': 'https://www.acgmx.com/',
    'user-agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/109.0.0.0 Safari/537.36'
}


def generate_image_struct() -> dict:
    return {
        'id': 0,
        'url': '',
        'title': '',
        'author': '',
        'tags': [],
        'data': None,
        'native': False,
    }

async def get_pix(keyword=['R-18'], img=1) -> list:
    logger.info('loading...')
    
    img = img if img < 11 else 1
    image_list = []
    image = generate_image_struct()
    
    async with AsyncClient() as client:
        url = 'https://api.acgmx.com/public/search'
        params = {
            'q': '%20'.join(keyword) if keyword else ['R-18',],
            'offset': 0
        }
        try:
            res = await client.get(
                url, params=params, headers=headers, timeout=30
            )
        except httpx.HTTPError as e:
            logger.warning(e)
            return [f'API异常{e}', False]

        try:
            logger.debug(data := res.json())

            if not data['illusts']:
                logger.info(e := f'图库中没有搜到关于{keyword}的图。')
                return [e, False]

            for _ in range(img if len(data['illusts']) > img else len(data['illusts'])):
                item = random.choice(data['illusts'])
                data['illusts'].remove(item)
                image = generate_image_struct()
                image['id'] = item['id']
                image['title'] = item['title']
                image['author'] = item['user']['name']
                for tag in item['tags']:
                    image['tags'].append(tag['name'])
                try:
                    if item['page_count'] == 1:
                        image['url'] = item['meta_single_page']['original_image_url']
                    else:
                        num = random.randint(0, item['page_count'] - 1)
                        image['url'] = item['meta_pages'][num]['image_urls']['original']
                except:
                    pass
                if image['url']:
                    image_list.append(image)

            content = [{
                'pid': i['id'],
                'url': i['url'],
                'caption': (
                    f'标题: {i["title"]}\n'
                    f'pid: {i["id"]}\n'
                    f'画师: {i["author"]}\n'
                    f'标签: {", ".join(i["tags"])}'
                )
            } for i in image_list]

            pics, status = await down_pic(content)

            logger.success('complete.')

            # if len(content) == 1:
            #     return [content[0]['url'], 1, content[0]['caption']]

            # media = [
            #     InputMediaPhoto(media=i['url'], caption=i['caption'])
            #     for i in content
            # ]
            # return [media, 2]

            if not pics:
                return ['\n'.join(status), False]
            if len(pics) == 1:
                return [pics[0], 1, '\n'.join(status) if status else '']

            media = [
                InputMediaPhoto(media=i[0], caption=i[1])
                for i in pics
            ]
            return [media, 2]

        except httpx.ProxyError as e:
            logger.warning(e)
            return [f'代理错误: {e}', False]
        except IndexError as e:
            logger.warning(e)
            return [f'图库中没有搜到关于{keyword}的图。', False]
        except:
            logger.warning(f'{exc_info()[0]}, {exc_info()[1]}')
            return [f'{exc_info()[0]} {exc_info()[1]}。', False]



async def down_pic(content) -> tuple[list, list]:
    async with AsyncClient() as client:
        headers = {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/109.0.0.0 Safari/537.36',
        }
        if not pixproxy:
            headers = {
                'Host': 'i.pximg.net',
                'Referer': 'https://www.pixiv.net/',
                'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/109.0.0.0 Safari/537.36'
            }
        pics, status = [], []
        for i in content:
            res = await client.get(
                url = (
                    i['url'].replace('https://i.pximg.net', pixproxy)
                    if pixproxy else i['url']
                ),
                headers=headers,
                timeout=30
            )
            if res.status_code == 200:
                logger.success(f'获取图片 {i["pid"]} 成功')
                pics.append([res.content, i['caption']])
            else:
                logger.error(sc := f'获取图片 {i["pid"]} 失败: {res.status_code}')
                status.append(sc)
            await res.aclose()
        return pics, status




from random import choice

class Look():
    @staticmethod
    def like():
        a = choice(["比基尼","破烂的衣服","JK","西装","东北大花袄","狼皮","虎皮裙","皮衣","军绿大衣","背心","校服","伊斯兰传统蒙面服饰","兜帽卫衣","孔乙己同款长衫","孔乙己时期同款短打","皇帝朝服","大臣朝服","中东悍匪同款服饰","杰哥同款短袖","阿伟同款短袖","牧民藏袍","军中训练服"])
        b = choice(["光腿","白色过膝袜","黑色丝袜","黑色渔网袜","校服裤子","大花袄配套花棉裤","牛仔裤","安全裤","蓝白胖次","军装配套裤子","西服裤子"])
        c = choice(["光脚","小皮鞋","运动鞋","老北京布鞋","军靴","鸿星尔克","兔子头棉拖","人字拖","脚镣","草鞋","木屐","高跟鞋","芭蕾舞鞋","皮鞋"])
        d = choice(["机械手","眼镜","翅膀","机械腿","渔夫帽","面具","眼罩","角","机械尾巴","伤痕","金箍","草帽","斗笠","蜂鸟胸针","大红花","金丝框眼镜","擦汗毛巾"])
        e = choice(["单手剑","长戟/枪","现代枪","斧头","镰刀","锤子","武士刀","弓","双剑","巨剑","扫把","伞","书","弩","钓鱼竿","铲子","旗帜","玩偶","棒球套装","盾牌（方形）","物理学圣剑","吉他","御币","爪子","匕首","砍刀","水果刀","法杖","电线杆","棺材","狼牙棒","铁链","手杖","电吉他","烧火棍","朗吉努斯之枪","西瓜刀","钥匙串","收租本","手机","左轮手枪","史密斯冲锋枪","皮鞭","锄头","粉笔","黑板擦","圆盾","神秘手提箱","拖拉机车钥匙","钢笔"])
        f = choice(["白色","红色","金色","黑色","粉色","灰色","绿色","蓝色"])
        g = choice(["披肩长发","中短发","双马尾","单马尾","丸子头","卷发","杂乱发型","麻花辫","寸头","光头","包租婆卷发","莫西干","蘑菇头"])
        h = choice(["黑色","褐色","蓝色","橙色","红色","绿色","灰色"])
        i = choice(["写轮眼","X形","❤形","正常","普普通通的圆形"])
        j = choice(["没有","A","B","C","D","E","F","G","H"])
        k = choice(["白色","黄色","棕色","黑色","灰色","绿色","蓝色","红色","粉色"])
        l = choice(["大高个","小胖墩儿","瘦小","普通","健硕","巨人","矮人"])
        m = choice(["内向","元气","阴郁","病娇","迷糊","笨蛋","知性","色欲","傲娇","暴躁","分裂型","理智","憨厚","怯懦","疯狂"])
        n = choice(["修女","女仆","护士","学生","人偶","天使","兽娘","恶魔","精灵","亡灵","法师","史莱姆","OL","萝莉","御姐","雇佣兵","律师","囚犯","算命先生","教师","清洁工","宿管阿姨","农民","剑灵","奴隶","坐台小姐","工人","水手","总统","包租婆"])
        
        return f"像一个戴着{d}上身{a}下身{b+c}手上拿着{e}的{f+g+h+i}瞳孔{j}cup{k}皮肤{l}体型的{m+n}。"
        



from telegram import Update
from telegram.ext import (
    ContextTypes,
    CommandHandler,
    filters
)

from remilia.log import logger
from remilia.config import SUPERUSERS
from .sys_cmd import shell, async_shell


def run(application):
    cmd_handler = CommandHandler(
        'cmd', cmd,
        filters.TEXT & filters.User(SUPERUSERS)
    )
    application.add_handler(cmd_handler)

    sh_handler = CommandHandler(
        'sh', sh,
        filters.TEXT & filters.User(SUPERUSERS)
    )
    application.add_handler(sh_handler)


async def cmd(update: Update, context: ContextTypes.DEFAULT_TYPE):
    logger.info(f'[{update.message.chat.type.upper()}]({update.message.chat_id}) {context._user_id}: {update.message.text}')
    opt = (
        context.args[0] + update.message.text.split(context.args[0], 1)[1]
        if context.args else ''
    )
    content = shell(opt)
    await update.message.reply_text(content)


async def sh(update: Update, context: ContextTypes.DEFAULT_TYPE):
    logger.info(f'[{update.message.chat.type.upper()}]({update.message.chat_id}) {context._user_id}: {update.message.text}')
    opt = (
        context.args[0] + update.message.text.split(context.args[0], 1)[1]
        if context.args else ''
    )
    content = await async_shell(opt)
    await update.message.reply_text(content)




from platform import system
from subprocess import Popen, PIPE
from asyncio import create_subprocess_shell
from asyncio.subprocess import PIPE as AsyncPIPE
from html import unescape


cmd_help = '''
调用系统命令行
⚠危险操作, 谨慎使用!

/cmd {命令}
For example:
/cmd echo "Hello World"
'''.strip()


shell_help = '''
调用系统命令行
(不支持Windows)
⚠危险操作, 谨慎使用!

/sh {命令}
For example:
/sh echo "Hello World"
'''.strip()


def shell(opt: str) -> str:

    if not opt:
        return '发送 /cmd -h 获取帮助'

    if opt == '-h':
        return cmd_help

    content = Popen(
        unescape(opt.strip()),
        stdin=PIPE,
        stdout=PIPE,
        stderr=PIPE,
        shell=True,
        universal_newlines=True
    ).communicate()

    if content == ('',''):
        msg = '执行完毕, 没有任何输出呢~'
    elif not content[1]:
        msg = f'stdout:\n{content[0]}\n>执行完毕'
    elif not content[0]:
        msg = f'stderr:\n{content[1]}'
    else:
        msg = f'stdout:\n{content[0]}\nstderr:\n{content[1]}\n>执行完毕'
    return msg


_win = ('windows', 'Windows', 'win32', 'Win32', 'win16', 'Win16')


async def async_shell(opt: str) -> str:
    for i in _win:
        if system() and (system() in i or i in system()):
            return '暂不支持Windows,\n请使用同步方法 `/cmd`'

    if not opt:
        return '发送 /sh -h 获取帮助'

    if opt == '-h':
        return shell_help

    content = await (await create_subprocess_shell(
        unescape(opt.strip()),
        stdin=AsyncPIPE,
        stdout=AsyncPIPE,
        stderr=AsyncPIPE
    )).communicate()

    if content == (b'', b''):
        msg = '\n执行完毕, 没有任何输出呢~'
    elif content[1] == b'':
        msg = f'\nstdout:\n{content[0].decode()}\n>执行完毕'
    elif content[0] == b'':
        msg = f'\nstderr:\n{content[1].decode()}'
    else :
        msg = (f'\nstdout:\n{content[0].decode()}'
               f'\nstderr:\n{content[1].decode()}'
               '\n>执行完毕')
    return msg



import logging
from typing import TYPE_CHECKING

import loguru

if TYPE_CHECKING:
    from loguru import Logger

logger: "Logger" = loguru.logger


class LoguruHandler(logging.Handler):  # pragma: no cover
    """logging 与 loguru 之间的桥梁，将 logging 的日志转发到 loguru。"""

    def emit(self, record: logging.LogRecord):
        try:
            level = logger.level(record.levelname).name
        except ValueError:
            level = record.levelno

        frame, depth = logging.currentframe(), 2
        while frame and frame.f_code.co_filename == logging.__file__:
            frame = frame.f_back
            depth += 1

        logger.opt(depth=depth, exception=record.exc_info).log(
            level, record.getMessage()
        )



default_format: str = (
    "<g>{time:MM-DD HH:mm:ss}</g> "
    "[<lvl>{level}</lvl>] "
    "<c><u>{name}</u></c> | "
    "<c>{function}:{line}</c>\n"
    "{message}"
)
"""默认日志格式"""

